@@ -118,9 +118,15 @@ from .utils import RTSLibError, modprobe, ignored
 from .target import Target
 from .utils import _get_auth_attr, _set_auth_attr
 
-version_attributes = set(["lio_version", "version"])
-discovery_auth_attributes = set(["discovery_auth"])
-target_names_excludes = version_attributes | discovery_auth_attributes
+excludes_list = [
+    # version_attributes
+    "lio_version", "version",
+    # discovery_auth_attributes
+    "discovery_auth",
+    # cpus_allowed_list_attributes
+    "cpus_allowed_list",
+]
+target_names_excludes = set(excludes_list)
 
 
 class _BaseFabricModule(CFSNode):
@@ -144,7 +150,8 @@ class _BaseFabricModule(CFSNode):
         self.name = name
         self.spec_file = "N/A"
         self._path = "%s/%s" % (self.configfs_dir, self.name)
-        self.features = ('discovery_auth', 'acls', 'auth', 'nps', 'tpgts')
+        self.features = ('discovery_auth', 'acls', 'auth', 'nps', 'tpgts',
+            'cpus_allowed_list')
         self.wwn_types = ('free',)
         self.kernel_module = "%s_target_mod" % self.name
 
@@ -220,6 +227,18 @@ class _BaseFabricModule(CFSNode):
             raise RTSLibError("Fabric module %s does not implement "
                               + "the %s feature" % (self.name, feature))
 
+    def _get_cpus_allowed_list(self):
+        self._check_self()
+        self._assert_feature('cpus_allowed_list')
+        path = "%s/cpus_allowed_list" % self.path
+        return fread(path)
+
+    def _set_cpus_allowed_list(self, allowed):
+        self._check_self()
+        self._assert_feature('cpus_allowed_list')
+        path = "%s/cpus_allowed_list" % self.path
+        fwrite(path, allowed)
+
     def clear_discovery_auth_settings(self):
         self._check_self()
         self._assert_feature('discovery_auth')
@@ -267,6 +286,11 @@ class _BaseFabricModule(CFSNode):
         self._assert_feature('discovery_auth')
         _set_auth_attr(self, *args, **kwargs)
 
+    cpus_allowed_list = \
+            property(_get_cpus_allowed_list,
+                     _set_cpus_allowed_list,
+                     doc="Set or get the cpus_allowed_list attribute.")
+
     discovery_enable_auth = \
             property(_get_discovery_enable_auth,
                      _set_discovery_enable_auth,
@@ -375,7 +399,35 @@ class Qla2xxxFabricModule(_BaseFabricModule):
     def wwns(self):
         for wwn_file in glob("/sys/class/fc_host/host*/port_name"):
             with ignored(IOError):
-                if not fread(os.path.dirname(wwn_file)+"/symbolic_name").startswith("fcoe"):
+                host = os.path.realpath(os.path.dirname(wwn_file))
+                device = host.rsplit('/', 3)[0]
+                driver = os.path.basename(os.path.realpath(device+"/driver"))
+                if driver == "qla2xxx":
+                    yield "naa." + fread(wwn_file)[2:]
+
+
+class EfctFabricModule(_BaseFabricModule):
+    def __init__(self):
+        super(EfctFabricModule, self).__init__('efct')
+        self.features = ("acls",)
+        self.wwn_types = ('naa',)
+        self.kernel_module = "efct"
+
+    def to_fabric_wwn(self, wwn):
+        # strip 'naa.' and add colons
+        return colonize(wwn[4:])
+
+    def from_fabric_wwn(self, wwn):
+        return "naa." + wwn.replace(":", "")
+
+    @property
+    def wwns(self):
+        for wwn_file in glob("/sys/class/fc_host/host*/port_name"):
+            with ignored(IOError):
+                host = os.path.realpath(os.path.dirname(wwn_file))
+                device = host.rsplit('/', 3)[0]
+                driver = os.path.basename(os.path.realpath(device+"/driver"))
+                if driver == "efct":
                     yield "naa." + fread(wwn_file)[2:]
 
 
@@ -419,7 +471,10 @@ class FCoEFabricModule(_BaseFabricModule):
     def wwns(self):
         for wwn_file in glob("/sys/class/fc_host/host*/port_name"):
             with ignored(IOError):
-                if fread(os.path.dirname(wwn_file)+"/symbolic_name").startswith("fcoe"):
+                host = os.path.realpath(os.path.dirname(wwn_file))
+                device = host.rsplit('/', 3)[0]
+                subsystem = os.path.basename(os.path.realpath(device+"/subsystem"))
+                if subsystem == "fcoe":
                     yield "naa." + fread(wwn_file)[2:]
 
 
@@ -465,6 +520,7 @@ fabric_modules = {
     "iscsi": ISCSIFabricModule,
     "loopback": LoopbackFabricModule,
     "qla2xxx": Qla2xxxFabricModule,
+    "efct": EfctFabricModule,
     "sbp": SBPFabricModule,
     "tcm_fc": FCoEFabricModule,
 #    "usb_gadget": USBGadgetFabricModule, # very rare, don't show
@@ -486,3 +542,10 @@ class FabricModule(object):
     def all(cls):
         for mod in six.itervalues(fabric_modules):
             yield mod()
+
+    @classmethod
+    def list_registered_drivers(cls):
+        try:
+            return os.listdir('/sys/module/target_core_mod/holders')
+        except OSError:
+            return []
